--- /dev/null	2012-08-18 10:37:10.000000000 +0000
+++ new/usr/src/uts/common/inet/dccp/dccp_misc.c	2012-08-18 10:37:10.266551197 +0000
@@ -0,0 +1,124 @@
+/*
+ * CDDL HEADER START
+ *
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License (the "License").
+ * You may not use this file except in compliance with the License.
+ *
+ * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
+ * or http://www.opensolaris.org/os/licensing.
+ * See the License for the specific language governing permissions
+ * and limitations under the License.
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
+ * If applicable, add the following below this CDDL HEADER, with the
+ * fields enclosed by brackets "[]" replaced with your own identifying
+ * information: Portions Copyright [yyyy] [name of copyright owner]
+ *
+ * CDDL HEADER END
+ */
+
+/*
+ * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright 2012 David Hoeppner.  All rights reserved.
+ */
+
+/*
+ * This file contains various helper functions.
+ */
+
+#include <sys/types.h>
+#include <sys/strlog.h>
+#include <sys/policy.h>
+#include <sys/strsun.h>
+
+#include <inet/common.h>
+#include <inet/dccp_impl.h>
+
+/*
+ * When a CPU is added, we need to allocate the per CPU stats struct.
+ */
+void
+dccp_stack_cpu_add(dccp_stack_t *dccps, processorid_t cpu_seqid)
+{
+	int	i;
+
+	if (cpu_seqid < dccps->dccps_sc_cnt) {
+		return;
+	}
+
+	for (i = dccps->dccps_sc_cnt; i <= cpu_seqid; i++) {
+		ASSERT(dccps->dccps_sc[i] == NULL);
+		dccps->dccps_sc[i] = kmem_zalloc(sizeof (dccp_stats_cpu_t),
+		    KM_SLEEP);
+	}
+
+	membar_producer();
+	dccps->dccps_sc_cnt = cpu_seqid + 1;
+}
+
+/*
+ * Diagnostic routine used to return a string associated with the dccp state.
+ */
+char *
+dccp_display(dccp_t *dccp, char *sup_buf, char format)
+{
+	conn_t		*connp;
+	in6_addr_t	local;
+	in6_addr_t	remote;
+	char		local_addrbuf[INET6_ADDRSTRLEN];
+	char		remote_addrbuf[INET6_ADDRSTRLEN];
+	char		priv_buf[INET6_ADDRSTRLEN * 2 + 80];
+	char		buf1[30];
+	char		*buf;
+	char		*cp;
+
+	if (sup_buf != NULL) {
+		buf = sup_buf;
+	} else {
+		buf = priv_buf;
+	}
+
+	if (dccp == NULL) {
+		return ("NULL_DCCP");
+	}
+
+	connp = dccp->dccp_connp;
+	switch (dccp->dccp_state) {
+	case DCCPS_CLOSED:
+		cp = "DCCP_CLOSED";
+		break;
+	default:
+		(void) mi_sprintf(buf1, "DCCPUnkState(%d)", dccp->dccp_state);
+		cp = buf1;
+		break;
+	}
+
+	switch (format) {
+	case DISP_ADDR_AND_PORT:
+		if (connp->conn_ipversion == IPV4_VERSION) {
+			IN6_IPADDR_TO_V4MAPPED(connp->conn_laddr_v4, &local);
+			IN6_IPADDR_TO_V4MAPPED(connp->conn_faddr_v4, &remote);
+		} else {
+			local = connp->conn_laddr_v6;
+			remote = connp->conn_faddr_v6;
+		}
+
+		(void) inet_ntop(AF_INET6, &local, local_addrbuf,
+		    sizeof (local_addrbuf));
+		(void) inet_ntop(AF_INET6, &remote, remote_addrbuf,
+		    sizeof (remote_addrbuf));
+		(void) mi_sprintf(buf, "[%s.%u, %s.%u] %s",
+		    local_addrbuf, ntohs(connp->conn_lport), remote_addrbuf,
+		    ntohs(connp->conn_fport), cp);
+		break;
+	case DISP_PORT_ONLY:
+	default:
+		(void) mi_sprintf(buf, "[%u, %u] %s",
+		    ntohs(connp->conn_lport), ntohs(connp->conn_fport), cp);
+		break;
+	}
+
+	return (buf);
+}
